//
//  main.cpp
//  rtsine
//
//  Created by Stanko Juzbasic on 1/19/16.
//


#include "RtAudio.h"
#include <iostream>
#include <cstdlib>
#include <cmath>
# ifndef twopi
# define twopi 6.283185307179586476925287
# endif
unsigned int sampleRate = 44100;
double       rad;

typedef struct {
    double f0;
    double f1;
    double j;
}uData;

//____________________________________________________________________________________
// Two-channel sine wave generator.
int sine( void *outputBuffer,
         void *inputBuffer,
         unsigned int nBufferFrames,
         double streamTime,
         RtAudioStreamStatus status,
         void *userData )
{
    unsigned int ch;

    double *buffer = (double *) outputBuffer;

    uData *data = (uData*)userData;
    
    unsigned int frame;
    double a = .5;
    
    if ( status )
        std::cout << "Stream underflow detected!" << std::endl;
    
    // Write interleaved audio data.
    for ( frame=0; frame < nBufferFrames; frame++ ) {
        double b = a*sin (rad * data->j * data->f0);  //wavelength0 in samples
        double c = a*sin (rad * data->j * data->f1);  //wavelength1 in samples
        for ( ch=0; ch<2; ch++ ) {
            buffer[2*frame] = b;
            buffer[2*frame+1] = c;
        }
        data->j = data->j + 1.;
    }   
    return 0;
}

//____________________________________________________________________________________
int main(int argc, const char * argv[])
{
    uData data;

    std::cout << "RTAUDIO TEST\n";
    
    RtAudio dac;
    if ( dac.getDeviceCount() < 1 ) {
        std::cout << "\nNo audio devices found!\n";
        exit( 0 );
    }
    RtAudio::StreamParameters parameters;
    parameters.deviceId = dac.getDefaultOutputDevice();
    parameters.nChannels = 2;
    parameters.firstChannel = 0;
    rad = twopi/sampleRate;
    unsigned int bufferFrames = 256; // 256 sample frames
    
    data.j = 0.; data.f0 = 523.1; data.f1 = 660;
    try {
        dac.openStream( &parameters, NULL, RTAUDIO_FLOAT64,
                       sampleRate, &bufferFrames, &sine, (void *)&data );
        dac.startStream();
    }
    catch ( RtAudioError& e ) {
        e.printMessage();
        exit( 0 );
    }
    
    char input;
    std::cout << "\nPlaying ... press <enter> to quit.\n";
    std::cin.get( input );
    try {
        // Stop the stream
        dac.stopStream();
    }
    catch (RtAudioError& e) {
        e.printMessage();
    }
    if ( dac.isStreamOpen() ) dac.closeStream();
    
    return 0;
}

